SystemOrganization addCategory: #VisuAule!
SystemOrganization addCategory: #'VisuAule-Objects'!
SystemOrganization addCategory: #'VisuAule-Objects-Tests'!
SystemOrganization addCategory: #'VisuAule-Example'!

TestCase subclass: #VABookAndPersonTest
	instanceVariableNames: 'book ralph eric joe'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VABookAndPersonTest methodsFor: 'running' stamp: 'CamiloGomez 7/6/2012 18:39'!
setUp
	
	book := VABook new name: 'design patterns'.
	ralph := VAPerson new name: 'Ralph Johnson'.
	eric := VAPerson new name: 'Eric Gamma'.
	book addAuthor: ralph; addAuthor: eric.
	
	joe := VAReader new.
	joe readBook: book.! !

!VABookAndPersonTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/5/2012 12:12'!
testPerson

	"The scenario we are here simulating is (person visualize)"
	
	| view |
	
	view := ROView new.
	
	ralph visualizeOn: view.
	eric visualizeOn: view.
	
	"view open"
	
	self assert: view elements size = 2! !

!VABookAndPersonTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/5/2012 12:12'!
testVisualizingBooksAndPersons

	"The scenario we are here simulating is (book new visualize)"
	
	| view |
		
	view := ROView new.
	
	self assert: view elements isEmpty.
	book visualizeOn: view.
	"view open"
	self assert: view elements size = 5.
	self assert: (view elementFromModel: book) notNil.
	self assert: (view elementFromModel: ralph) notNil.
	self assert: (view elementFromModel: eric) notNil.
	self assert: (view elementFromModel: joe) isNil.! !

TestCase subclass: #VAMovieTest
	instanceVariableNames: 'lordOfTheRings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VAMovieTest methodsFor: 'running' stamp: 'CamiloGomez 7/6/2012 16:52'!
setUp

	lordOfTheRings := VAMovie new.
	
	lordOfTheRings title: 'The Lord Of The Rings: The Fellowship Of The Ring'. 
	lordOfTheRings director: (VAPerson new name: 'Peter Jackson').
	lordOfTheRings duration: '3 hours'.
	lordOfTheRings country: 'New Zealand'.
	lordOfTheRings genre: 'Fantasy'.
	! !

!VAMovieTest methodsFor: 'tests' stamp: 'CamiloGomez 7/6/2012 16:58'!
testVisualizeMovie

	| view |
	
	view := ROView new.
	
	self assert: view elements isEmpty.
	
	lordOfTheRings visualizeOn: view.
	
	self assert: view elements size = 3.
	self assert: (view elementFromModel: lordOfTheRings) notNil.
	! !

TestCase subclass: #VAVisualizingPointTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects-Tests'!

!VAVisualizingPointTest methodsFor: 'testing scenarios' stamp: 'CamiloGomez 7/6/2012 16:37'!
testVisualizingPoint

	"The scenario we are here simulating is (2@4) visualize"
	
	| view |
	view := ROView new.
	
	self assert: view elements isEmpty.
	(2 @ 4) visualizeOn: view.
	(10 @ 10) visualizeOn: view.
	"view open."
	self assert: view elements size = 2.! !

!Browser methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 12:23'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | Browser open].! !

!Number methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/25/2012 16:27'!
elementsNotAdded 

	^ ROElement new.! !

!Number methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/25/2012 16:18'!
roassalElementFor: object
	| element |
	
	element := ROElement on: object.
	element + ROLabel + ROBorder.
	element translateTo: 5@5.
	element @ ROLightlyHighlightable @ RODraggable.
	
	^ element! !

!Number methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:38'!
shouldBeAddedAsElement
	^ false! !

!UndefinedObject methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:50'!
shouldBeAddedAsElement
	^ false! !

!UndefinedObject methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:15'!
shouldBeAddedAsElementOnEnter
	^ false! !

!OrderedCollection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 11/7/2012 22:31'!
interactionFor: elementForSelf on: view

	| el newView elementForSelfCopy cont newElements showMoreElements element1 |
		
		newElements := OrderedCollection new.
		
		super interactionFor: elementForSelf on: view.
	
		newView := self configureVisualization.
	
		showMoreElements := self showMoreElementsIn: view startingAt: 6.
		
		elementForSelf @ (ROMenuActivable new item: 'display elements' action:[

			self isSequenceable
			ifTrue: [
			(1 to: 5) do: [ :index | 
				element1 := (self at: index).
				element1 notNil
					ifTrue: [
						el := element1 roassalElementFor: element1.
						(elementForSelf model) addElement: el of: elementForSelf on: view.
						el translateTo: (elementForSelf position).
						newElements add: el.
					]
			].]
			ifFalse:[cont := 0.
			self do: [ :element | 
						cont < 5
						ifTrue: [
							element notNil
							ifTrue: [
								cont := cont + 1.
								el := element roassalElementFor: element.
								(elementForSelf model) addElement: el of: elementForSelf on: view.
								el translateTo: (elementForSelf position).
								newElements add: el.
							]
						].
			].].
			
			(self size) > 5
			ifTrue: [(elementForSelf model) addElement: (showMoreElements) of: elementForSelf on: view.
					(showMoreElements) translateTo: (elementForSelf position).
					newElements add: showMoreElements].
		
			"self setThisLayout: ROHorizontalTreeLayout on: newElements."
			self setThisLayout: ROHorizontalTreeLayout on: (view elements). 
			
		])
	
	! !

!ROEaselMorphic methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 17:12'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | ROEaselMorphic new open]. ! !

!Boolean methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:23'!
shouldBeAddedAsElement 
	^ false! !

!ThreadSafeTranscript methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/6/2012 15:55'!
interactionFor: element on: view

	super interactionFor: element on: view.
	
	element on: ROMouseLeftClick do: [ :event | Transcript open].! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/8/2012 21:07'!
addElement: el of: elementForSelf on: view

	" Add the element 'el', set the interaction for 'el', and add an Edge from 'elementForSelf' to 'el' on 'view' "

	"Add the interaction" 
	(el model) interactionFor: el on: view.
	
	"Add the element and an edge"
	view add: el.
	view add: ((ROEdge from: elementForSelf to: el)
		+ ROOrthoHorizontalLineShape ;
		yourself).! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 18:59'!
addReferencedObjectsOf: elementForSelf on: view

	" Add the variables of elementForSelf on view, unless it's a Collection, in which case, do handleCollection "

	"Go over all the Instance Variables of self" 
	self class allInstVarNames do: [ :varName |
		| el object |
		"Identify the object of varName"
		object := self instVarNamed: varName.
		object shouldBeAddedAsElement
			ifTrue: 
				[object isCollection
					"If it's a Collection, handle it as a Collection"
					ifTrue: [self handleCollectionIn: object of: elementForSelf on: view varName: varName]
					"If not, add the ROElement of object on view"
					ifFalse: [el := (elementForSelf model) roassalElementFor: object withName: (varName,' - ',(object class) asString). 
				 			(elementForSelf model) addElement: el of: elementForSelf on: view.]]] ! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 18:07'!
colorVA

	^ Color black! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 18:59'!
configureVisualization

	| view |
	
	view := ROView new.
	
	"We add some interaction for the view"
	view on: ROMouseLeftClick do: [ :event | ROZoomInMove new on: view]. "ZoomIn on Left Click"
	view on: ROMouseRightClick do: [ :event | ROZoomOutMove new on: view]. "ZoomOut on Right Click"
	view @ (RODraggableWithVelocity); @(RODraggable).
	
	^ view! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 17:58'!
elementsNotAdded

	" Return all the elements that will be poped up when the mouse is on top of elementForSelf "

	| otherElements elementObject newElement varNames varNamesElement otherElementsElement bothColumns varNameElement elementForCollection popupView |
	
	bothColumns := OrderedCollection new.
	varNamesElement := ROElement new.
	otherElementsElement := ROElement new.
	varNames := OrderedCollection new.
	otherElements := OrderedCollection new.
	varNameElement := ROElement new.
	newElement := ROElement new.
	popupView := ROView new.
	
	self class allInstVarNames do: [ :varName |
		| el object |
		object := self instVarNamed: varName.
		elementObject := self roassalOtherElementFor: object withName: (
																			(object asString) size > 30
																			ifTrue: [(((object asString) truncateTo: 30),'...')]
																			ifFalse: [(object asString)]).
		object shouldBeAddedAsElementOnEnter
		ifTrue: [ 
			varNameElement := ROElement on: varName.
			
			object shouldBeAddedAsElement
			ifTrue: [
				varNameElement + (ROLabel new color: Color red).
			]
			ifFalse: [
				varNameElement + ROLabel
			].
			
			varNames add: varNameElement.
			
			object isCollection & object shouldBeAddedAsElement
			ifFalse: [
				otherElements add: elementObject
			]	
			ifTrue: [
				(((object class) asString) at: 1) isVowel
				ifTrue:
				[elementForCollection := self roassalOtherElementFor: object withName: ('an ',(object class) asString).]
				ifFalse:
				[elementForCollection := self roassalOtherElementFor: object withName: ('a ',(object class) asString).].
				otherElements add: elementForCollection
			]		
		]
	].

	ROVerticalLineLayout on: varNames.
	ROVerticalLineLayout on: otherElements.
	
	varNamesElement addAll: varNames.
	otherElementsElement addAll: otherElements.
	
	bothColumns add: varNamesElement.
	bothColumns add: otherElementsElement.
	ROHorizontalLineLayout on: bothColumns. 

	varNames isEmpty
	ifTrue: [
		^ ROView new
	]
	ifFalse: [
		newElement addAll: bothColumns.
	
		newElement + (ROBorder new color: Color blue).
		newElement + (ROBox new color: Color white).
	
		popupView add: newElement.
	
		^ popupView
	]! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 17:55'!
handleCollectionIn: object of: elementForSelf on: view

	| el |
		
	el := (object) roassalElementFor: object.
	(elementForSelf model) addElement: el of: elementForSelf on: view.
 ! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 11/7/2012 22:38'!
handleCollectionIn: object of: elementForSelf on: view varName: varName

	| el |
	
	(varName = 'array')
	ifFalse: [
	el := (object) roassalElementFor: object withName: varName,' - ',(object class) asString.
	(elementForSelf model) addElement: el of: elementForSelf on: view.]! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 19:00'!
interactionFor: elementForSelf on: view

	" Set the interaction of 'elementForSelf' "

	| otherElements elementObject |

	elementForSelf @ (ROMenuActivable new item: 'inspect element' action: #inspect;
									item: 'browse element class' action: #browse;
									item: 'visualize object' action: 
									[
										(elementForSelf model) visualize.	
									]
								).
								
	elementForSelf @ (ROPopupView new view: ((elementForSelf model) elementsNotAdded)).
! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 18:09'!
roassalElementFor: object
	
	" Makes a ROElement with border and label on object "
	
	| element |
	
	element := ROElement on: object.
	element + (ROLabel new text: object class; color: self colorVA).
	element @ ROLightlyHighlightable @ RODraggable.
	element + (ROBorder new color: Color gray).
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 18:09'!
roassalElementFor: object withName: aString

	" Makes a ROElement with border and name aString "

	| element |
	
	element := ROElement on: object.
	element + (ROLabel new text: aString; color: self colorVA).
	element @ ROLightlyHighlightable @ RODraggable.
	element + (ROBorder new color: Color gray).
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/28/2012 17:00'!
roassalOtherElementFor: object

	" Makes a ROElement without border, with label on object "

	| element |
	
	element := ROElement on: object.
	element + ROLabel.
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/28/2012 17:00'!
roassalOtherElementFor: object withName: aString

	" Makes a ROElement without border, with label aString "

	| element |
	
	element := ROElement on: object.
	element + (ROLabel new text: aString; color: Color gray).
	
	^ element! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/28/2012 17:00'!
setThisLayout: layout on: elements

	" Set anyLayout of elements "

	layout new 
				translator: (ROSmoothLayoutTranslator new);
				applyOn: elements.! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:23'!
shouldBeAddedAsElement
	^ true! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 7/17/2012 17:12'!
shouldBeAddedAsElementOnEnter
	^ true! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 11/8/2012 17:59'!
viewLegend

	| legend colorLegendForCollection collection colorLegendForString string colorBoxes nameBoxes bothColumns colorLegendElement colorLegendStringElement nameLegendElement |
	
	legend := ROElement new.
	colorLegendForCollection := ROElement new + (ROBox new color: (OrderedCollection new color); extent: 15@15).
	colorLegendForString := ROElement new + (ROBox new color: (String new color); extent: 15@15).
	collection := ROElement new.
	collection + (ROLabel new text: 'Collection'; color: Color gray).
	string := ROElement new.
	string + (ROLabel new text: 'String'; color: Color gray).
	
	colorBoxes := OrderedCollection new.
	colorBoxes add: colorLegendForCollection; add: colorLegendForString.
	ROVerticalLineLayout on: colorBoxes. 
	
	colorLegendElement := ROElement new.
	colorLegendElement addAll: colorBoxes.
	
	nameBoxes := OrderedCollection new.
	nameBoxes add: collection; add: string.
	ROVerticalLineLayout on: nameBoxes.
	
	nameLegendElement := ROElement new.
	nameLegendElement addAll: nameBoxes.
	
	bothColumns := OrderedCollection new.
	bothColumns add: colorLegendElement; add: nameLegendElement.
	ROHorizontalLineLayout on: bothColumns. 
	
	legend addAll: bothColumns.
	
	legend + (ROBorder new color: Color gray) + (ROBox new color: Color white).
	
	legend @ RODraggable.
	
	legend translateTo: (5@350).
	
	^ legend. 
	! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 17:23'!
visualize

	" visualize self "

	| view obj stack |
	
	view := self configureVisualization.
	stack := ROViewStack new addFirst: view.
	
	self visualizeOn: view.
	
	"We set the title to the window"
	stack title: 'VisuAule'.
	
	"We add the legend"
	stack add: (self viewLegend).

	stack open.
! !

!Object methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/28/2012 17:55'!
visualizeOn: view

	" Visualize self on aView "

	 | elementForSelf |
	"view := view1 firstView."
	elementForSelf := self roassalElementFor: self.
	
	"We add the object"
	self interactionFor: elementForSelf on: view.
	view add: elementForSelf.
	
	"We add the referenced objects"
	self addReferencedObjectsOf: elementForSelf on: view.
	
	"We set a Layout, in this case, treeLayout"
	self setThisLayout: ROHorizontalTreeLayout on: (view elements).! !

Object subclass: #VABook
	instanceVariableNames: 'authors title name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VABook methodsFor: 'actions' stamp: 'CamiloGomez 6/27/2012 15:18'!
addAuthor: aPerson
	authors add: aPerson! !

!VABook methodsFor: 'initialize-release' stamp: 'CamiloGomez 6/27/2012 15:17'!
initialize
	super initialize.
	authors := OrderedCollection new.! !

!VABook methodsFor: 'accessing' stamp: 'CamiloGomez 6/27/2012 15:26'!
name: aString
	name := aString! !

Object subclass: #VAExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Example'!

!VAExample commentStamp: '<historical>' prior: 0!
List of examples!

!VAExample class methodsFor: 'as yet unclassified' stamp: 'CamiloGomez 11/9/2012 05:25'!
examplesSmallTalks2012

	Object new visualize.

	VAExample new visualizeVAMovie.

	VAExample visualizeOrderedCollection.

	Browser open visualize.

	Object visualize.

	VAExample visualizeOrderedCollectionVersion2.! !

!VAExample class methodsFor: 'as yet unclassified' stamp: 'CamiloGomez 11/7/2012 22:22'!
visualizeOrderedCollection

	| a |
	
	a := OrderedCollection new.
	a add: 1;
	   add: 'b';
	   add: (ROElement new);
	   add: (Browser new);
	   add: true;
	   visualize! !

!VAExample class methodsFor: 'as yet unclassified' stamp: 'CamiloGomez 11/9/2012 05:22'!
visualizeOrderedCollectionVersion2

	| a |
	
	a := OrderedCollection new.
	a add: 1;
	   add: 'b';
	   add: (ROElement new);
	   add: (Browser new);
	   add: true;
	   add: 'Element number 6';
        add: 'Universidad de Chile';
	   add: 27405;
	   add: (ROEaselMorphic new);
	   add: 'Final Element';
	   visualize! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 17:11'!
allTools

	"self new allTools"

	| view |
	
	view := ROView new.
	
	Browser new visualizeOn: view.
	Transcript visualizeOn: view.
	ROEaselMorphic new visualizeOn: view.
	
	view open.! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 12:20'!
browser
	"self new browser"

	Browser new visualize.! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 17:37'!
roassalEasel

	"self new roassalEasel"
	
	ROEaselMorphic new visualize.
	! !

!VAExample methodsFor: 'layout' stamp: 'CamiloGomez 7/6/2012 12:20'!
simpleLayout

	"self new simpleLayout"

	"Not implemented yet"! !

!VAExample methodsFor: 'open tools' stamp: 'CamiloGomez 7/6/2012 15:55'!
transcript

	"self new transcript"
	
	Transcript visualize.! !

!VAExample methodsFor: 'as yet unclassified' stamp: 'CamiloGomez 7/17/2012 17:25'!
visualizeVADirector 

	"self new visualizeVADirector"

	| director view |
	
	view := ROView new.
	
	director := VADirector new name: 'Francis Ford Coppola'.
	
	director visualizeOn: view.	
	
	view open.! !

!VAExample methodsFor: 'va objects' stamp: 'CamiloGomez 8/9/2012 16:24'!
visualizeVAMovie

	"self new visualizeVAMovie"

	| movie movie2 movie3 movie4 movie5 view director |
	
	view := ROView new.
	movie := VAMovie new.
	movie2 := VAMovie new.
	movie3 := VAMovie new.
	movie4 := VAMovie new.
	movie5 := VAMovie new.
	director := (VADirector new name: 'Francis Ford Coppola'; directedMovie: movie; directedMovie: movie2; directedMovie: movie3 ; directedMovie: movie4; directedMovie: movie5 ).
	
	movie title: 'The Godfather'; 
		    director: director; 
		    duration: '175 minutes'; 
		    country: 'USA'; 
		    genre: 'Suspense'. 
	
	movie2 title: 'The Godfather 2';
			director: director;
			duration: '200 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie3 title: 'The Godfather 3';
			director: director;
			duration: '163 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie4 title: 'Apocalypse Now';
			director: director;
			duration: '153 minutes';
			country: 'USA';
			genre: 'Drama'.
			
	movie5 title: 'The Outsiders';
			director: director;
			duration: '91 minutes';
			country: 'USA';
			genre: 'Drama'.
	
	director visualize.! !

Object subclass: #VAMovie
	instanceVariableNames: 'title director duration country genre'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
country: aString

	country := aString! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:04'!
director: aDirector

	director := aDirector! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 16:25'!
duration: aTime

	duration := aTime! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
genre: aString

	genre := aString! !

!VAMovie methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:53'!
initialize

	super initialize.
	
	title := ''.
	director := VADirector new.
	duration := ''.
	country := ''.
	genre := ''.! !

!VAMovie methodsFor: 'accesing' stamp: 'CamiloGomez 7/6/2012 18:05'!
title: aString

	title := aString! !

Object subclass: #VAPerson
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

VAPerson subclass: #VADirector
	instanceVariableNames: 'directedMovies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VADirector methodsFor: 'actions' stamp: 'CamiloGomez 7/6/2012 18:03'!
directedMovie: aMovie

	directedMovies add: aMovie.! !

!VADirector methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:50'!
initialize

	super initialize.
	directedMovies := OrderedCollection new.! !

!VADirector methodsFor: 'as yet unclassified' stamp: 'CamiloGomez 7/18/2012 15:40'!
shouldBeAddedAsElementOnEnter
	^ false! !

!VAPerson methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/17/2012 17:27'!
initialize
	super initialize.
	
	name := ''.! !

!VAPerson methodsFor: 'actions' stamp: 'CamiloGomez 6/27/2012 15:25'!
name: aString
	name := aString! !

VAPerson subclass: #VAReader
	instanceVariableNames: 'readBooks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VisuAule-Objects'!

!VAReader methodsFor: 'initialize-release' stamp: 'CamiloGomez 7/6/2012 18:03'!
initialize

	super initialize.
	readBooks := OrderedCollection new.! !

!VAReader methodsFor: 'actions' stamp: 'CamiloGomez 7/6/2012 18:02'!
readBook: aBook
	readBooks add: aBook! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/28/2012 09:03'!
color

	^ Color blue! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 18:10'!
colorVA

	^ Color blue! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/27/2012 16:26'!
elementsNotAdded 

	^ ROView new.! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/27/2012 16:30'!
interactionFor: elementForSelf on: view

	| otherElements elementObject |

	elementForSelf @ (ROMenuActivable new item: 'inspect element' action: #inspect;
									item: 'browse element class' action: #browse";
									item: 'visualize object' action: 
									[
										(elementForSelf model) visualize.	
									]"
								).
								
	elementForSelf @ (ROPopupView new view: ((elementForSelf model) elementsNotAdded)).! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 18:10'!
roassalElementFor: object
	| element |
	
	element := ROElement on: object.
	element + (ROLabel new text: (' ''',object asString,''' '); color: self colorVA).
	element translateTo: 5@5.
	element @ ROLightlyHighlightable @ RODraggable.
	element + (ROBorder new color: Color gray).
	
	^ element! !

!String methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 6/27/2012 15:39'!
shouldBeAddedAsElement
	^ false! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/28/2012 09:03'!
color

	^ Color green! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 18:10'!
colorVA

	^ Color green! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 9/6/2012 15:12'!
elementsNotAdded

	| bothColumns newElement varSize nameSize popupView |
	
	bothColumns := OrderedCollection new.
	newElement := ROElement new.
	varSize := ROElement new.
	nameSize := ROElement new.
	popupView := ROView new.
	
	nameSize := ((ROElement on: 'size') + ROLabel).
	varSize := (self roassalOtherElementFor: self size).
	
	bothColumns add: nameSize.
	bothColumns add: varSize.
	ROHorizontalLineLayout on: bothColumns.
	
	newElement addAll: bothColumns.
	
	newElement + (ROBorder new color: Color blue).
	newElement + (ROBox new color: Color white).

	popupView add: newElement.
	
	^ popupView.
	
! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/3/2012 20:13'!
interactionFor: elementForSelf on: view

	| el newView elementForSelfCopy cont newElements showMoreElements element1 |
		
		newElements := OrderedCollection new.
		
		super interactionFor: elementForSelf on: view.
	
		newView := self configureVisualization.
	
		showMoreElements := self showMoreElementsIn: view startingAt: 6.
		
		elementForSelf @ (ROMenuActivable new item: 'display elements' action:[

			self isSequenceable
			ifTrue: [
			(1 to: 5) do: [ :index | 
				element1 := (self at: index).
				element1 notNil
					ifTrue: [
						el := element1 roassalElementFor: element1.
						(elementForSelf model) addElement: el of: elementForSelf on: view.
						el translateTo: (elementForSelf position).
						newElements add: el.
					]
			].]
			ifFalse:[cont := 0.
			self do: [ :element | 
						cont < 5
						ifTrue: [
							element notNil
							ifTrue: [
								cont := cont + 1.
								el := element roassalElementFor: element.
								(elementForSelf model) addElement: el of: elementForSelf on: view.
								el translateTo: (elementForSelf position).
								newElements add: el.
							]
						].
			].].
			
			(self size) > 5
			ifTrue: [(elementForSelf model) addElement: (showMoreElements) of: elementForSelf on: view.
					(showMoreElements) translateTo: (elementForSelf position).
					newElements add: showMoreElements].
		
			"self setThisLayout: ROHorizontalTreeLayout on: newElements."
			self setThisLayout: ROHorizontalTreeLayout on: (view elements). 
			
		])
	
	! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 19:23'!
showMoreElement

	| element |
	
	element := ROElement new.
	element + (ROLabel new text: 'show more (5)'; color: Color gray).
	element @ ROLightlyHighlightable .
	
	element on: ROMouseLeftClick do: [:event | ].
	
	^ element.! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/2/2012 19:54'!
showMoreElementsIn: view

	| element element1 el elementForSelf cont |
	
	element := ROElement new.
	element + (ROLabel new text: 'show more (5)').
	element @ ROLightlyHighlightable.
	
	"elementForSelf := self roassalElementFor: self."
	
	element on: ROMouseLeftClick do: [:event | 
		self isSequenceable 
		ifTrue: [
			(6 to: 10) do: [ :index | 
				element1 := (self at: index).
				element1 notNil
					ifTrue: [
						elementForSelf := ((view elements) at: 1). "Arreglar seriamente!!"
						el := element1 roassalElementFor: element1.
						(elementForSelf model) addElement: el of: elementForSelf on: view.
						el translateTo: (elementForSelf position).
					]
			].]
		ifFalse:[cont := 5.
			self do: [ :elementA | 
						cont < 10
						ifTrue: [
							element notNil
							ifTrue: [
								cont := cont + 1.
								el := element roassalElementFor: element.
								(elementForSelf model) addElement: el of: elementForSelf on: view.
								el translateTo: (elementForSelf position).
								"newElements add: el."
							]
						].
			].].
	self setThisLayout: ROHorizontalTreeLayout on: (view elements).
	].
	
	^ element.! !

!Collection methodsFor: '*visuaule-objects' stamp: 'CamiloGomez 10/3/2012 20:14'!
showMoreElementsIn: view startingAt: firstIndex

	| element element1 el elementForSelf cont |
	
	element := ROElement new.
	element + (ROLabel new text: 'show more (5)').
	element @ ROLightlyHighlightable.
	
	"elementForSelf := self roassalElementFor: self."
	
	element on: ROMouseLeftClick do: [:event | 
		self isSequenceable 
		ifTrue: [
			(firstIndex to: firstIndex + 4) do: [ :index | 
				element1 := (self at: index).
				element1 notNil
					ifTrue: [
						elementForSelf := ((view elements) at: 1). "Arreglar seriamente!!"
						el := element1 roassalElementFor: element1.
						(elementForSelf model) addElement: el of: elementForSelf on: view.
						el translateTo: (elementForSelf position).
					]
			].]
		ifFalse:[
			cont := firstIndex.
			self do: [ :elementA | 
						(cont < (firstIndex + 4)) & ((self at: cont) = elementA)
						ifTrue: [
							element notNil
							ifTrue: [
								cont := cont + 1.
								elementForSelf := ((view elements) at: 1). "Arreglar seriamente!!"
								el := element roassalElementFor: element.
								(elementForSelf model) addElement: el of: elementForSelf on: view.
								el translateTo: (elementForSelf position).
								"newElements add: el."
							]
						].
			].].
	self setThisLayout: ROHorizontalTreeLayout on: (view elements).
	].
	
	^ element.! !
